@@ -10,6 +10,8 @@ import copy
 import ctypes
 import json
 import os
+from functools import partial
+import platform
 import importlib.resources
 import re
 import threading
@@ -24,19 +26,20 @@ from .chat import ChatManager, config_conversation, config_template_prompts
 
 # The ollama-chat back-end API WSGI application class
 class OllamaChat(chisel.Application):
-    __slots__ = ('config', 'chats', 'downloads')
+    __slots__ = ('config', 'xorigin', 'chats', 'downloads')
 
 
-    def __init__(self, config_path):
+    def __init__(self, config_path, xorigin=False):
         super().__init__()
         self.config = ConfigManager(config_path)
+        self.xorigin = xorigin
         self.chats = {}
         self.downloads = {}
 
-        # Add the chisel documentation application
+        # Back-end documentation
         self.add_requests(chisel.create_doc_requests())
 
-        # Add the APIs
+        # Back-end APIs
         self.add_request(create_template)
         self.add_request(delete_conversation)
         self.add_request(delete_conversation_exchange)
@@ -60,42 +63,37 @@ class OllamaChat(chisel.Application):
         self.add_request(stop_model_download)
         self.add_request(update_template)
 
-        # Add the ollama-chat statics
-        self.add_static(
-            'index.html',
-            'text/html; charset=utf-8',
-            (('GET', None), ('GET', '/')),
-            'The Ollama Chat application HTML'
-        )
-        self.add_static(
-            'ollamaChat.bare',
-            'text/plain; charset=utf-8',
-            (('GET', None),),
-            'The Ollama Chat application'
-        )
-        self.add_static(
-            'ollamaChatConversation.bare',
-            'text/plain; charset=utf-8',
-            (('GET', None),),
-            'The Ollama Chat application conversation page'
-        )
-        self.add_static(
-            'ollamaChatModels.bare',
-            'text/plain; charset=utf-8',
-            (('GET', None),),
-            'The Ollama Chat application models page'
-        )
-        self.add_static(
-            'ollamaChatTemplate.bare',
-            'text/plain; charset=utf-8',
-            (('GET', None),),
-            'The Ollama Chat application template page'
-        )
+        # Front-end statics
+        self.add_static('index.html', urls=(('GET', None), ('GET', '/')))
+        self.add_static('ollamaChat.bare')
+        self.add_static('ollamaChatConversation.bare')
+        self.add_static('ollamaChatModels.bare')
+        self.add_static('ollamaChatTemplate.bare')
 
 
-    def add_static(self, filename, content_type, urls, doc, doc_group='Ollama Chat Statics'):
+    def add_static(self, filename, urls=(('GET', None),), doc_group='Ollama Chat Statics'):
+        content_type = _CONTENT_TYPES.get(os.path.splitext(filename)[1], 'text/plain; charset=utf-8')
         with importlib.resources.files('ollama_chat.static').joinpath(filename).open('rb') as fh:
-            self.add_request(chisel.StaticRequest(filename, fh.read(), content_type, urls, doc, doc_group))
+            self.add_request(chisel.StaticRequest(filename, fh.read(), content_type, urls, doc_group=doc_group))
+
+
+    def __call__(self, environ, start_response):
+        if self.xorigin:
+            return super().__call__(environ, partial(_start_response_xorigin, start_response))
+        return super().__call__(environ, start_response)
+
+
+def _start_response_xorigin(start_response, status, headers):
+    headers_inner = list(headers)
+    headers_inner.append(('Access-Control-Allow-Origin', '*'))
+    start_response(status, headers_inner)
+
+
+_CONTENT_TYPES = {
+    '.css': 'text/css; charset=utf-8',
+    '.js': 'text/javascript; charset=utf-8',
+    '.html': 'text/html; charset=utf-8'
+}
 
 
 # The ollama-chat configuration context manager
@@ -103,9 +101,6 @@ class ConfigManager:
     __slots__ = ('config_path', 'config_lock', 'config')
 
 
-    DEFAULT_MODEL = 'llama3.2:latest'
-
-
     def __init__(self, config_path):
         self.config_path = config_path
         self.config_lock = threading.Lock()
@@ -136,11 +131,6 @@ class ConfigManager:
             self.config_lock.release()
 
 
-    @classmethod
-    def get_model(cls, config):
-        return config.get('model', cls.DEFAULT_MODEL)
-
-
 # The model download manager class
 class DownloadManager():
     __slots__ = ('app', 'model', 'status', 'completed', 'total', 'stop')
@@ -189,31 +179,27 @@ with importlib.resources.files('ollama_chat.static').joinpath('ollamaChat.smd').
 @chisel.action(name='getConversations', types=OLLAMA_CHAT_TYPES)
 def get_conversations(ctx, unused_req):
     with ctx.app.config() as config:
-        return {
-            'model': ConfigManager.get_model(config),
+        response = {
             'conversations': [
                 {
                     'id': conversation['id'],
                     'model': conversation['model'],
-                    'title': conversation['title']
+                    'title': conversation['title'],
+                    'generating': conversation['id'] in ctx.app.chats
                 }
                 for conversation in config['conversations']
             ],
-            'templates': [
-                _create_template_info(template)
-                for template in (config.get('templates') or ())
+            'templates': [] if 'templates' not in config else [
+                {
+                    'id': template['id'],
+                    'title': template['title']
+                }
+                for template in config['templates']
             ]
         }
-
-
-def _create_template_info(template):
-    template_info = {
-        'id': template['id'],
-        'title': template['title']
-    }
-    if 'name' in template:
-        template_info['name'] = template['name']
-    return template_info
+        if 'model' in config:
+            response['model'] = config['model']
+        return response
 
 
 @chisel.action(name='setModel', types=OLLAMA_CHAT_TYPES)
@@ -270,7 +256,7 @@ def move_template(ctx, req):
 def delete_template(ctx, req):
     with ctx.app.config(save=True) as config:
         id_ = req['id']
-        templates = config['templates'] or []
+        templates = config.get('templates') or []
         ix_tmpl = next((ix for ix, tmpl in enumerate(templates) if tmpl['id'] == id_), None)
         if ix_tmpl is None:
             raise chisel.ActionError('UnknownTemplateID')
@@ -311,13 +297,11 @@ def start_conversation(ctx, req):
             title = f'{title[:max_title_len - len(title_suffix)]}{title_suffix}'
 
         # Create the new conversation object
+        model = req.get('model', config.get('model'))
+        if model is None:
+            raise chisel.ActionError('NoModel')
         id_ = str(uuid.uuid4())
-        conversation = {
-            'id': id_,
-            'model': ConfigManager.get_model(config),
-            'title': title,
-            'exchanges': []
-        }
+        conversation = {'id': id_, 'model': model, 'title': title, 'exchanges': []}
 
         # Add the new conversation to the application config
         config['conversations'].insert(0, conversation)
@@ -352,13 +336,11 @@ def start_template(ctx, req):
             raise chisel.ActionError(error, message)
 
         # Create the new conversation object
+        model = req.get('model', config.get('model'))
+        if model is None:
+            raise chisel.ActionError('NoModel')
         id_ = str(uuid.uuid4())
-        conversation = {
-            'id': id_,
-            'model': ConfigManager.get_model(config),
-            'title': title,
-            'exchanges': []
-        }
+        conversation = {'id': id_, 'model': model, 'title': title, 'exchanges': []}
 
         # Add the new conversation to the application config
         config['conversations'].insert(0, conversation)
@@ -396,10 +378,13 @@ def get_conversation(ctx, req):
         if conversation is None:
             raise chisel.ActionError('UnknownConversationID')
 
+        # Add the generating status
+        conversation = copy.deepcopy(conversation)
+        conversation['generating'] = id_ in ctx.app.chats
+
         # Return the conversation
         return {
-            'conversation': copy.deepcopy(conversation),
-            'generating': id_ in ctx.app.chats
+            'conversation': conversation
         }
 
 
@@ -518,10 +503,6 @@ def regenerate_conversation_exchange(ctx, req):
 
 @chisel.action(name='getModels', types=OLLAMA_CHAT_TYPES)
 def get_models(ctx, unused_req):
-    # Get the current model
-    with ctx.app.config() as config:
-        current_model = ConfigManager.get_model(config)
-
     # Get the Ollama models
     try:
         models = ollama.list()['models'] or ()
@@ -533,34 +514,36 @@ def get_models(ctx, unused_req):
         {
             'id': model.model,
             'name': model.model[:model.model.index(':')],
-            'parameters': _parse_parameter_size(model.details.parameter_size),
+            'parameters': _parse_parameter_size(ctx, model.details.parameter_size),
             'size': model.size,
             'modified': model.modified_at
         }
         for model in models
     ]
 
-    # Create the downloading models response
-    downloading_models = []
-    for model_id, download_manager in ctx.app.downloads.items():
-        download = {
-            'id': model_id,
-            'status': download_manager.status,
-            'completed': download_manager.completed
+    with ctx.app.config() as config:
+        # Create the downloading models response
+        downloading_models = []
+        for model_id, download_manager in ctx.app.downloads.items():
+            download = {
+                'id': model_id,
+                'status': download_manager.status,
+                'completed': download_manager.completed
+            }
+            if download_manager.total:
+                download['size'] = download_manager.total
+            downloading_models.append(download)
+
+        response = {
+            'models': sorted(response_models, key=lambda model: model['id']),
+            'downloading': sorted(downloading_models, key=lambda model: model['id'])
         }
-        if download_manager.total:
-            download['size'] = download_manager.total
-        downloading_models.append(download)
-
-    return {
-        'model': current_model,
-        'models': sorted(response_models, key=lambda model: model['id']),
-        'downloading': sorted(downloading_models, key=lambda model: model['id'])
-    }
+        if 'model' in config:
+            response['model'] = config['model']
+        return response
 
 
-# Helper function to parse parameter sizes
-def _parse_parameter_size(parameter_size):
+def _parse_parameter_size(ctx, parameter_size):
     value = float(parameter_size[:-1])
     unit = parameter_size[-1]
     if unit == 'B':
@@ -569,7 +552,9 @@ def _parse_parameter_size(parameter_size):
         return int(value * 1000000)
     elif unit == 'K':
         return int(value * 1000)
-    raise ValueError(f'Unrecognized parameter size: {parameter_size}')
+
+    ctx.log.warning(f'Invalid parameter size "{parameter_size}"')
+    return 0
 
 
 @chisel.action(name='downloadModel', types=OLLAMA_CHAT_TYPES)
@@ -593,13 +578,13 @@ def delete_model(unused_ctx, req):
 @chisel.action(name='getSystemInfo', types=OLLAMA_CHAT_TYPES)
 def get_system_info(unused_ctx, unused_req):
     # Compute the total memory
-    if os.name == 'nt':
+    if platform.system() == "Windows": # pragma: no cover
         memory_status = MEMORYSTATUSEX()
         # pylint: disable-next=invalid-name, attribute-defined-outside-init
         memory_status.dwLength = ctypes.sizeof(MEMORYSTATUSEX)
         ctypes.windll.kernel32.GlobalMemoryStatusEx(ctypes.byref(memory_status))
         total_memory = memory_status.ullTotalPhys
-    else:
+    else: # pragma: no cover
         # pylint: disable-next=no-member, useless-suppression
         total_memory = os.sysconf("SC_PHYS_PAGES") * os.sysconf("SC_PAGE_SIZE")
 
